/****
 *
 * Exemplo: Separando um String em Partes (Tokens)
 *
 * Seo: 3.4.9
 *
 ****/

#include <stdio.h>
#include <string.h>

/****
*
* ObtemTokens(): Separa um string em partes (tokens)
*
* Parmetros:
*     str (entrada/sada) - o string que ser separado
*                           ou NULL para continuar uma
*                           separao anterior
*     sep (entrada) - string contendo os separadores de tokens
*
* Retorno: Endereo do prximo token do string, se este
*          existir, ou NULL, em caso contrrio
*
* Observao: Esta funo NO  funcionalmente equivalente 
*             funo strtok() da biblioteca padro de C
*
****/
char *ObtemTokens(char *str, char const *sep)
{
   static char *proximoToken; /* Apontar para o prximo */
                              /* token, se ele existir   */
   char        *s, /* Apontar para o string no qual a */
                   /* busca pelo token ser efetuada   */
               *inicio = NULL; /* Guardar o incio */
                               /* do token corrente */

   if (str != NULL)
      proximoToken = str;

   if (proximoToken == NULL)
      return NULL; /* No h mais token neste string */

      /* A funo strpbrk() retorna o endereo do  */
      /* primeiro caractere de 'sep' encontrado em */
      /* 'proximoToken'. Se tal caractere no for  */
      /* encontrado, essa funo retorna NULL.     */
   if ((s = strpbrk(proximoToken, sep)) != NULL) {
         /* Termina o token corrente na posio em */
         /* que se encontra o separador encontrado */
      *s  = '\0';

         /* Guarda o incio do token corrente */
      inicio = proximoToken;

         /* O prximo token iniciar no caractere */
         /* que segue o separador encontrado      */
      proximoToken = ++s;
   } else if (*proximoToken) {
         /* No foi encontrado nenhum separador, mas o string  */
         /* corrente no  vazio e seu endereo ser retornado */
      inicio = proximoToken;

         /* No haver mais token na prxima chamada */
      proximoToken = NULL;
   }

   return inicio;
}

int main(void)
{
   char string[] = "um,dois,,quatro,cinco";
   char separadores[] = " ,"; /* Note o espao em branco */
   char *token;
   int  i = 0;

   printf("\nString a ser separado em tokens: "
          "\"%s\"\n", string);
   printf("\nOs tokens sao:\n");

   token = ObtemTokens(string, separadores);

   while (token) {
      printf("\tToken %d: %s\n", ++i,
             *token ? token : "<vazio>");
      token = ObtemTokens(NULL, separadores);
   }

   return 0;
}

/****************** Resultado do Programa *****************

String a ser separado em tokens: "um,dois,,quatro,cinco"

Os tokens sao:
        Token 1: um
        Token 2: dois
        Token 3: <vazio>
        Token 4: quatro
        Token 5: cinco

**********************************************************/
