/****
 *
 * Ttulo: Leitura de Nomes
 *
 * Autor: Ulysses de Oliveira
 *
 * Data de Criao: 17/05/2012
 * ltima modificao: 18/05/2012
 *
 * Descrio: L um string e verifica se ele contm apenas letras
 *            e espaos em branco
 *
 * Entrada: Um string contendo apenas letras e espaos
 *
 * Sada: V. Exemplo de execuo
 *
 ****/

/************************ Includes ************************/

#include <stdio.h>   /* printf() */
#include <string.h>  /* Processamento de strings */
#include <ctype.h>   /* isalpha() */

/****************** Constantes Simblicas *****************/

#define MAX_NOME 20 /* Nmero mximo de caracteres num nome */

/******************* Aluses de Funes *******************/

extern int LimpaBuffer(void);
extern int LeString(char *ar, int nElementos);
extern int LeNome(char *nome, int tam);

/***************** Definies de Funes ******************/

/****
 *
 * LimpaBuffer(): Limpa o buffer associado ao meio de
 *                entrada padro (stdin)
 *
 * Parmetros: Nenhum
 *
 * Retorno: O nmero de caracteres descartados, com
 *          exceo do caractere '\n'.
 *
 ****/
int LimpaBuffer(void)
{
   int carLido, /* Armazena cada caractere lido */
       nCarLidos = 0; /* Conta o nmero de */
                      /* caracteres lidos  */

      /* L e descarta cada caractere lido at    */
      /* encontrar '\n' ou getchar() retornar EOF */
   do {
      carLido = getchar(); /* L um caractere */
      ++nCarLidos; /* Mais um caractere foi lido */
   } while ((carLido != '\n') && (carLido != EOF));

      /* O ltimo caractere lido foi '\n' ou  */
      /* EOF e no deve ser considerado sobra */
   return nCarLidos - 1;
}

/****
 *
 * LeString(): L um string via teclado deixando o buffer
 *             zerado
 *
 * Parmetros:
 *      ar[] (sada): array que armazenar o string lido
 *      nElementos (entrada): nmero mximo de elementos que
 *                            sero armazenados no array,
 *                            incluindo '\0'
 *
 * Retorno: O nmero de caracteres que o usurio digitou a
 *          mais, sem incluir '\n'
 *
 * Observaes:
 *      1. "Ler um string" significa ler caracteres,
 *         armazen-los num array e acrescentar o
 *         caractere '\0' ao final para que o array
 *         contenha um string.
 *      2. O nmero mximo de caracteres lidos ser
 *         nElementos - 1, porque se deve deixar um
 *         espao sobressalente para o caractere '\0'.
 *      3. Se o caractere '\n' for lido, ele no far
 *         parte do string resultante.
 *
 ****/
int LeString(char *ar, int nElementos)
{
   char *p; /* Usado para remover '\n' do string */
   int   nCarDeixados = 0; /* Conta o nmero de     */
                           /* caracteres excedentes */
                           /* no buffer             */

      /* L no mximo nElementos - 1 caracteres ou at   */
      /* encontrar '\n'. Quando fgets() retorna NULL,    */
      /* ocorreu erro ou tentativa de leitura alm do    */
      /* final do arquivo. Nesses casos, o lao continua */
      /* efetuando novas tentativas de leitura.          */
   while ( fgets(ar, nElementos, stdin) == NULL )  {
      printf( "\a\n\t>>> %s", "Erro de leitura. "
              "Tente novamente\n\t> " );
   }

      /* Faz p apontar para o caractere que      */
      /* antecede o caractere terminal do string */
   p = strchr(ar, '\0') - 1;

      /* Se o caractere '\n' foi lido, remove-o do   */
      /* string. Caso contrrio, remove-o do buffer. */
   if (*p == '\n') {
         /* Caractere '\n' faz parte do string */
      *p = '\0'; /* Sobrescreve '\n' com '\0' */
   } else { /* Usurio digitou caracteres demais */
      nCarDeixados = LimpaBuffer();
   }

      /* Retorna o nmero de caracteres que o usurio */
      /* digitou alm do esperado, sem incluir '\n'   */
   return nCarDeixados;
}

/****
 *
 * LeNome(): L um string via teclado e garante que ele s
 *           contm letras e espaos em branco
 *
 * Parmetros: nome (sada) - array que conter o string lido
 *             tam (entrada) - nmero de elementos do array
 *
 * Retorno: Nmero de caracteres digitados alm do limite permitido
 *
 * Observaes:
 *      1. Caracteres digitados alm do limite especificado
 *         sero ignorados
 *      2. O nico espao em branco permitido num nome  ' '
 *      3. Esta funo no testa se o string digitado 
 *         constitudo apenas por espaos em branco
 *
 ****/
int LeNome(char *nome, int tam)
{
   int   remanescentes; /* Nmero de caracteres digitados */
                        /* alm do permitido              */
   char *p; /* Usado para testar a validade do string */

   while (1) {
      printf("\nDigite o nome (max = %d letras):\n\t> ", tam-1);
      remanescentes = LeString(nome, tam);

         /* Verifica se o string lido  vazio */
      if (!*nome) {
         printf("\a\n\t>> Um nome nao pode ser vazio <<\n");
         continue; /* Salta o restante do lao */
      }

            /***************************************/
            /* Verifica se o string lido s contm */
            /* letras e espaos em branco          */
            /***************************************/

         /* O ponteiro p ser usado para testar o string */
      p = nome;

         /* O lao encerra quando p apontar para o caractere */
         /* terminal do string ou quando for encontrado um   */
         /* caractere que nem  letra nem espao em branco   */
      while (*p) {
            /* Se for encontrado um caractere que no  letra */
            /* ou espao em branco encerra este lao          */
         if( !isalpha(*p) && *p != ' ' ) {
            break;
         }

         p++; /* Aponta para o prximo caractere */
      }

         /* Se p apontar para o caractere terminal do */
         /* string, este foi aprovado no teste acima  */
      if (!*p) {
         break; /* String s contm letras e espaos */
      } else { /* String foi reprovado no teste */
         printf("\a\n\t>>> Um nome deve conter apenas <<<"
                "\n\t>>> letras e espacos em branco <<<\n");
      }
   }

   return remanescentes;
}

/****
 *
 * main(): Testa funo LeNome()
 *
 * Parmetros: Nenhum
 *
 * Retorno: Zero
 *
 ****/
int main(void)
{
   char umNome[MAX_NOME + 1];
   int  resto; /* Caracteres excedentes na leitura */

      /* Apresenta o programa */
   printf( "\n\t>>> Este programa le nomes contendo apenas"
           "\n\t>>> letras e espacos em branco\n" );

      /* L o nome e atribui a 'resto' o nmero */
      /* de caracteres excedentes na leitura    */
   resto = LeNome(umNome, MAX_NOME + 1);

      /* Apresenta o nome que o programa efetivamente leu */
   printf("\n\t>>> O nome aceito foi: %s\n", umNome);

      /* Verifica se o usurio digitou caracteres demais */
   if (resto) {
      printf("\n\t>>> %d caracteres foram desprezados\n", resto);
   }

   return 0;
}

/******************* Exemplo de Execuo *******************

        >>> Este programa le nomes contendo apenas
        >>> letras e espacos em branco

Digite o nome (max = 20 letras):
        > Micro$oft

        >>> Um nome deve conter apenas <<<
        >>> letras e espacos em branco <<<

Digite o nome (max = 20 letras):
        >

        >> Um nome nao pode ser vazio <<

Digite o nome (max = 20 letras):
        > Oftalmotorrinolaringologista

        >>> O nome aceito foi: Oftalmotorrinolaring

        >>> 8 caracteres foram desprezados

***********************************************************/
