/****
 *
 * Ttulo: Checando a Validade de PIS/PASEP
 *
 * Autor: Ulysses de Oliveira
 *
 * Data de Criao: 14/06/2012
 * ltima modificao: 14/06/2012
 *
 * Descrio: Verifica se um string constitui um nmero de PIS/PASEP
 *            vlido
 *
 * Entrada: Um string lido via teclado
 *
 * Sada: Mensagem informando se o string constitui um nmero de
 *        PIS/PASEP vlido
 *
 ****/

/************************ Includes ************************/

#include <stdio.h>   /* printf() */
#include <string.h>  /* Processamento de strings */
#include <ctype.h>   /* isalpha() */

/****************** Constantes Simblicas *****************/

   /* Nmero de dgitos num nmero de PIS/PASEP */
#define TAMANHO_PIS 11

/******************* Aluses de Funes *******************/

extern int LimpaBuffer(void);
extern int LeString(char *ar, int nElementos);
extern char *LeIdentidade( char *id, int tamArray,
                           const char *prompt);

/***************** Definies de Funes ******************/

/****
 *
 * LimpaBuffer(): Limpa o buffer associado ao meio de
 *                entrada padro (stdin)
 *
 * Parmetros: Nenhum
 *
 * Retorno: O nmero de caracteres descartados, com
 *          exceo do caractere '\n'.
 *
 ****/
int LimpaBuffer(void)
{
   int carLido, /* Armazena cada caractere lido */
       nCarLidos = 0; /* Conta o nmero de */
                      /* caracteres lidos  */

      /* L e descarta cada caractere lido at    */
      /* encontrar '\n' ou getchar() retornar EOF */
   do {
      carLido = getchar(); /* L um caractere */
      ++nCarLidos; /* Mais um caractere foi lido */
   } while ((carLido != '\n') && (carLido != EOF));

      /* O ltimo caractere lido foi '\n' ou  */
      /* EOF e no deve ser considerado sobra */
   return nCarLidos - 1;
}

/****
 *
 * LeString(): L um string via teclado deixando o buffer
 *             zerado
 *
 * Parmetros:
 *      ar[] (sada): array que armazenar o string lido
 *      nElementos (entrada): nmero mximo de elementos que
 *                            sero armazenados no array,
 *                            incluindo '\0'
 *
 * Retorno: O nmero de caracteres que o usurio digitou a
 *          mais, sem incluir '\n'
 *
 * Observaes:
 *      1. "Ler um string" significa ler caracteres,
 *         armazen-los num array e acrescentar o
 *         caractere '\0' ao final para que o array
 *         contenha um string.
 *      2. O nmero mximo de caracteres lidos ser
 *         nElementos - 1, porque se deve deixar um
 *         espao sobressalente para o caractere '\0'.
 *      3. Se o caractere '\n' for lido, ele no far
 *         parte do string resultante.
 *
 ****/
int LeString(char *ar, int nElementos)
{
   char *p; /* Usado para remover '\n' do string */
   int   nCarDeixados = 0; /* Conta o nmero de     */
                           /* caracteres excedentes */
                           /* no buffer             */

      /* L no mximo nElementos - 1 caracteres ou at   */
      /* encontrar '\n'. Quando fgets() retorna NULL,    */
      /* ocorreu erro ou tentativa de leitura alm do    */
      /* final do arquivo. Nesses casos, o lao continua */
      /* efetuando novas tentativas de leitura.          */
   while ( fgets(ar, nElementos, stdin) == NULL )  {
      printf( "\a\n\t>>> %s", "Erro de leitura. "
              "Tente novamente\n\t> " );
   }

      /* Faz p apontar para o caractere que      */
      /* antecede o caractere terminal do string */
   p = strchr(ar, '\0') - 1;

      /* Se o caractere '\n' foi lido, remove-o do   */
      /* string. Caso contrrio, remove-o do buffer. */
   if (*p == '\n') {
         /* Caractere '\n' faz parte do string */
      *p = '\0'; /* Sobrescreve '\n' com '\0' */
   } else { /* Usurio digitou caracteres demais */
      nCarDeixados = LimpaBuffer();
   }

      /* Retorna o nmero de caracteres que o usurio */
      /* digitou alm do esperado, sem incluir '\n'   */
   return nCarDeixados;
}

/****
 *
 * LeIdentidade(): L um nmero de identificao via teclado
 *
 * Parmetros: id[] (sada) - array que conter o nmero de
 *                            identificao lido
 *             tamArray (entrada) - nmero de elementos
 *                                  do array id[]
 *             prompt (entrada) - string que representa
 *                                a poro inicial do
 *                                prompt a ser apresentado
 *
 * Retorno: O endereo do array apontado por id
 *
 * Observao: Assume-se que o tamanho do nmero de
 *             identificao  igual ao tamanho do array
 *             id[] que a armazenar menos um.
 *
 ****/
char *LeIdentidade( char *id, int tamArray,
                    const char *prompt )
{
   int   resto; /* Caracteres excedentes que */
                /* possam ter sido digitados */
   char *p;

      /* O lao while encerra quando */
      /* for lido um string vlido   */
   while (1) {
         /* Apresenta prompt */
      printf( "\n%s com exatamente %d digitos:\n\t> ",
              prompt, tamArray - 1 );

         /* L entrada do usurio */
      resto = LeString(id, tamArray);

         /* Verifica se o usurio digitou */
         /* caracteres em excesso         */
      if (resto) {
         printf("\a\n\t>>> Excesso de caracteres <<<\n");
         continue; /* Faz nova tentativa */
      }

         /* Verifica se foram digitados menos caracteres */
         /* do que esperado. Note o uso do operador de   */
         /* converso explcita (int) para converter o   */
         /* valor retornado por strlen(), que  do tipo  */
         /* size_t, e, assim, evitar a comparao de um  */
         /* valor sem sinal e outro com sinal.           */
      if ( (int)strlen(id) != tamArray - 1 ) {
         printf("\a\n\t>>> Numero incompleto <<<\n");
         continue; /* Faz nova tentativa */
      }

         /*************************************/
         /*                                   */
         /* Neste ponto, o tamanho do nmero  */
         /* est correto. Resta testar se ele */
         /* s contm dgitos.                */
         /*                                   */
         /*************************************/

         /* O ponteiro p ser usado para testar o string */
      p = id;

         /* O lao encerra quando p apontar para o     */
         /* caractere terminal do string ou quando for */
         /* encontrado um caractere que no  dgito   */
      while (*p) {
            /* Se for encontrado um caractere que */
            /* no  dgito encerra este lao     */
         if(!isdigit(*p)) {
            break; /* Encerra o lao interno */
         }

         p++; /* Aponta para o prximo caractere */
      }

         /* Se p apontar para o caractere terminal do */
         /* string, este foi aprovado no teste acima  */
      if (!*p) {
         break; /* Nmero de identificao vlido */
      } else { /* Nmero de identificao invlido */
         printf("\a\n\t>>> \'%c\' nao e' digito <<<\n", *p);
      }
   }

   return id;
}

/****
 *
 * main(): L um string e verifica se ele constitui um nmero de
 *         PIS/PASEP vlido
 *
 * Parmetros: Nenhum
 *
 * Retorno: Zero
 *
 ****/
int main(void)
{
   char pis[TAMANHO_PIS + 1];
   int  i, soma = 0,
        pesos[] = {3, 2, 9, 8, 7, 6, 5, 4, 3, 2};

      /* Apresenta o programa */
   printf( "\n\t>>> Este programa verifica se um numero"
           "\n\t>>> de PIS/PASEP esta' correto.\n" );

      /* L o nmero de PIS/PASEP. A funo      */
      /* LeIdentidade() garante que o string    */
      /* do tamanho correto e s contm dgitos. */
   LeIdentidade(pis, TAMANHO_PIS + 1, "Digite o PIS/PASEP");

      /* Agora, calcula-se a soma ponderada dos dgitos */
   for (i = 0; i < TAMANHO_PIS - 1; ++i) {
      soma = soma + pesos[i]*(pis[i] - '0');
   }

      /* Neste ponto, i  igual a TAMANHO_PIS - 1, que  */
      /*  o ndice do ltimo dgito (i.e., o dgito    */
      /* verificador). Ento, checa-se se o valor do    */
      /* dgito verificador  igual a 11 menos o resto  */
      /* da diviso da soma ponderada obtida por 11.    */
   if (pis[i] - '0' == TAMANHO_PIS - soma%TAMANHO_PIS) {
      printf("\n\t>>> O numero de PIS e' valido\n");
   } else {
      printf("\n\t>>> O numero de PIS e' INVALIDO\n");
   }

   return 0;
}

/******************* Exemplo de Execuo *******************

        >>> Este programa verifica se um numero
        >>> de PIS/PASEP esta' correto.

Digite o PIS/PASEP com exatamente 11 digitos:
        > 18434219765

        >>> O numero de PIS e' valido

***********************************************************/
