/****
 *
 * Ttulo: Removendo Espaos Excedentes em Strings
 *
 * Autor: Ulysses de Oliveira
 *
 * Data de Criao: 16/06/2012
 * ltima modificao: 16/06/2012
 *
 * Descrio: Transforma espaos em branco mltiplos de um string
 *            em espaos simples
 *
 * Entrada: No h
 *
 * Sada: Um string antes e depois de ter seus espaos adicionais
 *        comprimidos
 *
 ****/

/************************ Includes ************************/

#include <stdio.h>   /* printf()   */
#include <string.h>  /* Processamento de strings */

/****************** Constantes Simblicas *****************/

   /* Tamanhos do array que armazenar os strings */
#define TAMANHO_ARRAY 50

/******************* Aluses de Funes *******************/

extern int LimpaBuffer(void);
extern int LeString(char *ar, int nElementos);
extern char *ComprimeEspacos(char *s);

/***************** Definies de Funes ******************/

/****
 *
 * LimpaBuffer(): Limpa o buffer associado ao meio de
 *                entrada padro (stdin)
 *
 * Parmetros: Nenhum
 *
 * Retorno: O nmero de caracteres descartados, com
 *          exceo do caractere '\n'.
 *
 ****/
int LimpaBuffer(void)
{
   int carLido, /* Armazena cada caractere lido */
       nCarLidos = 0; /* Conta o nmero de */
                      /* caracteres lidos  */

      /* L e descarta cada caractere lido at    */
      /* encontrar '\n' ou getchar() retornar EOF */
   do {
      carLido = getchar(); /* L um caractere */
      ++nCarLidos; /* Mais um caractere foi lido */
   } while ((carLido != '\n') && (carLido != EOF));

      /* O ltimo caractere lido foi '\n' ou  */
      /* EOF e no deve ser considerado sobra */
   return nCarLidos - 1;
}

/****
 *
 * LeString(): L um string via teclado deixando o buffer
 *             zerado
 *
 * Parmetros:
 *      ar[] (sada): array que armazenar o string lido
 *      nElementos (entrada): nmero mximo de elementos que
 *                            sero armazenados no array,
 *                            incluindo '\0'
 *
 * Retorno: O nmero de caracteres que o usurio digitou a
 *          mais, sem incluir '\n'
 *
 * Observaes:
 *      1. "Ler um string" significa ler caracteres,
 *         armazen-los num array e acrescentar o
 *         caractere '\0' ao final para que o array
 *         contenha um string.
 *      2. O nmero mximo de caracteres lidos ser
 *         nElementos - 1, porque se deve deixar um
 *         espao sobressalente para o caractere '\0'.
 *      3. Se o caractere '\n' for lido, ele no far
 *         parte do string resultante.
 *
 ****/
int LeString(char *ar, int nElementos)
{
   char *p; /* Usado para remover '\n' do string */
   int   nCarDeixados = 0; /* Conta o nmero de     */
                           /* caracteres excedentes */
                           /* no buffer             */

      /* L no mximo nElementos - 1 caracteres ou at   */
      /* encontrar '\n'. Quando fgets() retorna NULL,    */
      /* ocorreu erro ou tentativa de leitura alm do    */
      /* final do arquivo. Nesses casos, o lao continua */
      /* efetuando novas tentativas de leitura.          */
   while ( fgets(ar, nElementos, stdin) == NULL )  {
      printf( "\a\n\t>>> %s", "Erro de leitura. "
              "Tente novamente\n\t> " );
   }

      /* Faz p apontar para o caractere que      */
      /* antecede o caractere terminal do string */
   p = strchr(ar, '\0') - 1;

      /* Se o caractere '\n' foi lido, remove-o do   */
      /* string. Caso contrrio, remove-o do buffer. */
   if (*p == '\n') {
         /* Caractere '\n' faz parte do string */
      *p = '\0'; /* Sobrescreve '\n' com '\0' */
   } else { /* Usurio digitou caracteres demais */
      nCarDeixados = LimpaBuffer();
   }

      /* Retorna o nmero de caracteres que o usurio */
      /* digitou alm do esperado, sem incluir '\n'   */
   return nCarDeixados;
}

/****
 *
 * ComprimeEspacos(): Transforma espaos em branco mltiplos de
 *                    um string em espaos simples
 *
 * Parmetros:
 *      s (entrada/sada): string cujos espaos sero comprimidos
 *
 * Retorno: Endereo do string recebido como parmetro, que  o
 *          mesmo endereo do string resultante da operao
 *
 * Observao: O nico espao em branco considerado  ' ',
 *             obtido com a barra do teclado.
 *
 ****/
char *ComprimeEspacos(char *s)
{
   char *p; /* Ponteiro auxiliar */
   int   i, /* Usado como ndice de caracteres */
         espacoEncontrado; /* Indica se o ltimo caractere  */
                           /* encontrado  espao em branco */

      /* Faz p apontar para o incio do string. Ao final, */
      /* o ponteiro s apontar para o resultado da        */
      /* operao, pois ele no  alterado.               */
   p = s;

   i = 0; /* ndice do primeiro caractere a ser acessado */

      /* Nenhum espao em branco foi encontrado ainda */
   espacoEncontrado = 0;

      /* O lao encerra quando p apontar para '\0' */
   while(*p) {
      /*                                                  */
      /* Se o caractere corrente for espao em branco e o */
      /* antecedente tambm, faz p apontar para o prximo */
      /* caractere. Caso contrrio e o caractere corrente */
      /* no for espao em branco, atualiza a varivel    */
      /* 'espacoEncontrado' indicando que o ltimo        */
      /* caractere acessado no foi espao. Se o caractere*/
      /* corrente foi espao, mas o anterior no era      */
      /* espao, atribui ' ' ao caractere corrente e faz  */
      /* a varivel 'espacoEncontrado' indicar esse fato. */
      /*                                                  */

      if(*p == ' ' && espacoEncontrado) {
            /* O caractere corrente  espao em branco */
            /* e o anterior tambm foi. Portanto, faz  */
            /* p apontar para o prximo caractere.     */
         p++;
      } else {
            /* Indica se o ltimo caractere lido */
            /* foi um espao em branco ou no    */
         espacoEncontrado = *p == ' ' ? 1 : 0;

            /* O ponteiro s apontar para o string  */
            /* resultante. Portanto, copia-se o     */
            /* caractere correntemente apontado por */
            /* p e incrementam-se p e o ndice i.   */
         s[i] = *p;
         p++;
         i++;
      }
   }

   s[i] = '\0'; /* Termina o string resultante da operao */

   return s; /* Retorna o endereo do string resultante */
}

/****
 *
 * main(): Testa a funo ComprimeEspacos()
 *
 * Parmetros: Nenhum
 *
 * Retorno: Zero
 *
 ****/
int main(void)
{
   char str[TAMANHO_ARRAY];

      /* Apresenta o programa */
   printf( "\n\t>>> Este programa le textos e comprime"
           "\n\t>>> seus espacos em branco excedentes."
           "\n\t>>> Digite apenas [ENTER] para encerra-lo.\n" );

      /* O lao encerra quando o usurio digita apenas [ENTER] */
   while (1) {
         /* L o texto */
      printf( "\n\t>>> Introduza um texto (max = %d "
              "caracteres):\n\t\t> ", TAMANHO_ARRAY - 1 );
      (void) LeString(str, TAMANHO_ARRAY);

         /* Se apenas [ENTER] foi digitado, encerra o lao */
      if (!*str) {
         break; /* Usurio digitou apenas [ENTER] */
      }

         /* Apresenta o string tal  como ele foi introduzido */
      printf( "\n\t>>> String original:\n\t    \"%s\"", str );

         /* Comprime os possveis espaos */
         /* excedentes do string          */
      ComprimeEspacos(str);

         /* Apresenta o string sem espaos excedentes */
      printf( "\n\n\t>>> String sem espacos excedentes:"
              "\n\t    \"%s\"\n", str );
   }

   printf( "\n\t>>> Obrigado por usar este programa.\n");

   return 0;
}

/******************* Exemplo de Execuo ******************

        >>> Este programa le textos e comprime
        >>> seus espacos em branco excedentes.
        >>> Digite apenas [ENTER] para encerra-lo.

        >>> Introduza um texto (max = 49 caracteres):
                > String    com     espacos   excedentes

        >>> String original:
            "String    com     espacos   excedentes"

        >>> String sem espacos excedentes:
            "String com espacos excedentes"

        >>> Introduza um texto (max = 49 caracteres):
                > String sem espacos excedentes

        >>> String original:
            "String sem espacos excedentes"

        >>> String sem espacos excedentes:
            "String sem espacos excedentes"

        >>> Introduza um texto (max = 49 caracteres):
                > [ENTER]

        >>> Obrigado por usar este programa.

***********************************************************/
