/****
 *
 * Ttulo: Removendo Caracteres Duplicados de um String
 *
 * Autor: Ulysses de Oliveira
 *
 * Data de Criao: 05/04/2013
 * ltima modificao: 05/04/2013
 *
 * Descrio: Remove caracteres duplicados de strings
 *
 * Entrada: Strings introduzidos via teclado
 *
 * Sada: Mensagens apresentando os resultados das operaes
 *
 ****/

/*********************** Includes *************************/

#include <stdio.h>   /* printf()                 */
#include <string.h>  /* Processamento de strings */

/***************** Constantes Simblicas ******************/

   /* Tamanho do array que armazena os strings */
#define TAM_ARRAY 10

/******************* Aluses de Funes *******************/

extern int LimpaBuffer(void);
extern int LeString(char *ar, int nElementos);
extern char *RemoveCaracteresDuplicados(char *str);

/***************** Definies de Funes ******************/

/****
 *
 * LimpaBuffer(): Limpa o buffer associado ao meio de
 *                entrada padro (stdin)
 *
 * Parmetros: Nenhum
 *
 * Retorno: O nmero de caracteres descartados, com
 *          exceo do caractere '\n'.
 *
 ****/
int LimpaBuffer(void)
{
   int carLido, /* Armazena cada caractere lido */
       nCarLidos = 0; /* Conta o nmero de */
                      /* caracteres lidos  */

      /* L e descarta cada caractere lido at    */
      /* encontrar '\n' ou getchar() retornar EOF */
   do {
      carLido = getchar(); /* L um caractere */
      ++nCarLidos; /* Mais um caractere foi lido */
   } while ((carLido != '\n') && (carLido != EOF));

      /* O ltimo caractere lido foi '\n' ou  */
      /* EOF e no deve ser considerado sobra */
   return nCarLidos - 1;
}

/****
 *
 * LeString(): L um string via teclado deixando o buffer
 *             zerado
 *
 * Parmetros:
 *      ar[] (sada): array que armazenar o string lido
 *      nElementos (entrada): nmero mximo de elementos que
 *                            sero armazenados no array,
 *                            incluindo '\0'
 *
 * Retorno: O nmero de caracteres que o usurio digitou a
 *          mais, sem incluir '\n'
 *
 * Observaes:
 *      1. "Ler um string" significa ler caracteres,
 *         armazen-los num array e acrescentar o
 *         caractere '\0' ao final para que o array
 *         contenha um string.
 *      2. O nmero mximo de caracteres lidos ser
 *         nElementos - 1, porque se deve deixar um
 *         espao sobressalente para o caractere '\0'.
 *      3. Se o caractere '\n' for lido, ele no far
 *         parte do string resultante.
 *
 ****/
int LeString(char *ar, int nElementos)
{
   char *p; /* Usado para remover '\n' do string */
   int   nCarDeixados = 0; /* Conta o nmero de     */
                           /* caracteres excedentes */
                           /* no buffer             */

      /* L no mximo nElementos - 1 caracteres ou at   */
      /* encontrar '\n'. Quando fgets() retorna NULL,    */
      /* ocorreu erro ou tentativa de leitura alm do    */
      /* final do arquivo. Nesses casos, o lao continua */
      /* efetuando novas tentativas de leitura.          */
   while ( fgets(ar, nElementos, stdin) == NULL )  {
      printf( "\a\n\t>>> %s", "Erro de leitura. "
              "Tente novamente\n\t> " );
   }

      /* Faz p apontar para o caractere que      */
      /* antecede o caractere terminal do string */
   p = strchr(ar, '\0') - 1;

      /* Se o caractere '\n' foi lido, remove-o do   */
      /* string. Caso contrrio, remove-o do buffer. */
   if (*p == '\n') {
         /* Caractere '\n' faz parte do string */
      *p = '\0'; /* Sobrescreve '\n' com '\0' */
   } else { /* Usurio digitou caracteres demais */
      nCarDeixados = LimpaBuffer();
   }

      /* Retorna o nmero de caracteres que o usurio */
      /* digitou alm do esperado, sem incluir '\n'   */
   return nCarDeixados;
}

/****
 *
 * RemoveCaracteresDuplicados(): Remove de um string duplicatas
 *                               de caracteres
 *
 * Parmetros:
 *      str (entrada/sada) - o string
 *
 * Retorno: Endereo do string modificado ou NULL quando o
 *          parmetro for NULL
 *
 * Observao: Uma ocorrncia de caractere  considerada uma
 *             duplicata quando ela  encontrada mais de uma
 *             vez num string. Assim, a primeira ocorrncia
 *             de um caractere nunca  duplicata.
 *
 ****/
char *RemoveCaracteresDuplicados(char *str)
{
   char *p;
   int   guardaCar; /* Guarda um caractere */

   p = str; /* Faz p apontar para o incio do string */

      /* O lao encerra quando todos os caracteres */
      /* do string tiverem sido acessados (i.e.,   */
      /* quando p apontar para o caractere '\0')   */
   while (*p) {
         /* Guarda o caractere para o qual p aponta. */
         /* Se esse caractere no for duplicata, ele */
         /* ser armazenado de volta no string.      */
      guardaCar = *p;

         /* Armazena um caractere terminal na posio do  */
         /* caractere que foi guardado de modo que do     */
         /* endereo apontado por str at aquele apontado */
         /* por p se tenha um novo string (i.e., o string */
         /* original  dividido em dois strings)          */
      *p = '\0';

         /* Verifica se o caractere guardado encontra-se */
         /* no primeiro string. Se for o caso, ele  um  */
         /* caractere duplicado que deve ser removido.   */
      if (strchr(str, guardaCar)) {
            /* O caractere para o qual p apontava deve ser */
            /* removido. Ento, concatena-se a poro do   */
            /* string, que comea no endereo p + 1, com o */
            /* primeiro string mencionado acima. Assim, os */
            /* dois strings so  unificados novamente.     */
         strcat(str, p + 1);
      } else {
            /* O caractere guardado no  repetido. Assim, */
            /* coloca-se ele de volta no string e faz-se p */
            /* apontar para o prximo caractere. Os dois   */
            /* strings so unificados novamente.           */
         *p = guardaCar; /* Devolve o caractere ao string */
                         /* unificando-o novamente        */
         ++p; /* Passa para o prximo caractere */
      }
   }

      /* O ponteiro str continua apontando */
      /* para o incio do string           */
   return str;
}

/****
 *
 * main(): Remove caracteres duplicados de cada string lido
 *
 * Parmetros: Nenhum
 *
 * Retorno: Zero
 *
 ****/
int main(void)
{
   char  ar[TAM_ARRAY];

   printf( "\n\t>>> Este programa remove caracteres duplicados"
           "\n\t>>> de cadeias de caracteres."
           "\n\t>>> Digite apenas [ENTER] para encerra-lo.\n" );

      /* O lao encerra quando o usurio */
      /* digitar apenas [ENTER]          */
   while (1) {
         /* L o string */
      printf("\n\t>>> Digite uma sequencia de caracteres "
             "\n\t>>> (max = %d caracteres): ", TAM_ARRAY);
      LeString(ar, TAM_ARRAY);

      if (!*ar) {/* Se foi lido apenas [ENTER], encerra o lao */
         break;
      }

         /* Apresenta o string aps eventuais remoes */
      printf( "\n\t>>> String sem duplicatas de caracteres:"
              " %s\n", RemoveCaracteresDuplicados(ar) );
   }

   printf( "\n\t>>> Obrigado por usar este programa.\n");

   return 0;
}

/******************* Exemplo de Execuo ******************

        >>> Este programa remove caracteres duplicados
        >>> de cadeias de caracteres.
        >>> Digite apenas [ENTER] para encerra-lo.

        >>> Digite uma sequencia de caracteres
        >>> (max = 10 caracteres): banana

        >>> String sem duplicatas de caracteres: ban

        >>> Digite uma sequencia de caracteres
        >>> (max = 10 caracteres): [ENTER]

        >>> Obrigado por usar este programa.

**********************************************************/
