/****
 *
 * Exemplo: Funes com Retorno de Estruturas
 *
 * Seo: 4.3.2
 *
 * NB: Este programa contm um erro intencional
 *
 ****/

#include <stdio.h>   /* Entrada e sada          */
#include <string.h>  /* Processamento de strings */

#define MAX_NOME 30

typedef  struct  {
            char  nome[MAX_NOME];
            int   dia, mes, ano;
         } tRegistro;

/****
 *
 * LimpaBuffer(): Limpa o buffer associado ao meio de
 *                entrada padro (stdin)
 *
 * Parmetros: Nenhum
 *
 * Retorno: O nmero de caracteres descartados, com
 *          exceo do caractere '\n'.
 *
 ****/
int LimpaBuffer(void)
{
   int carLido, /* Armazena cada caractere lido */
       nCarLidos = 0; /* Conta o nmero de */
                      /* caracteres lidos  */

      /* L e descarta cada caractere lido at    */
      /* encontrar '\n' ou getchar() retornar EOF */
   do {
      carLido = getchar(); /* L um caractere */
      ++nCarLidos; /* Mais um caractere foi lido */
   } while ((carLido != '\n') && (carLido != EOF));

      /* O ltimo caractere lido foi '\n' ou  */
      /* EOF e no deve ser considerado sobra */
   return nCarLidos - 1;
}

/****
 *
 * LeInteiro(): L um nmero inteiro via teclado
 *              e deixa o buffer intacto
 *
 * Parmetros: Nenhum
 *
 * Retorno: O nmero inteiro lido
 *
 ****/
int LeInteiro(void)
{
   int num, /* O nmero lido */
       teste, /* Valor retornado por scanf() */
       nResto = 0; /* Nmero de caracteres */
                   /* excedentes           */

   /* Desvia para c se o valor for invlido */
inicio:
      /* Tenta ler um valor vlido */
   teste = scanf("%d", &num);

      /* Se no ocorreu erro de leitura ou de final */
      /* de arquivo, h caracteres remanescentes    */
      /* que precisam ser removidos                 */
   if (teste != EOF) {
      nResto = LimpaBuffer();
   }

      /* Enquanto o valor retornado por scanf()  */
      /* no indicar que um valor vlido foi     */
      /* lido continua tentando obter esse valor */
   while(teste != 1) {
      printf("\a\n\t>>> O valor digitado e' invalido. "
                      "Tente novamente\n\t> " );
      goto inicio; /* No causa dano algum */
   }

      /* Repreende o usurio se ele digitou demais */
   if (nResto == 1) {
      printf("\t>>> Um caractere foi descartado\n");
   } else if (nResto > 1) {
      printf("\t>>> %d caracteres foram descartados\n",
             nResto);
   }

      /* O valor retornado certamente  vlido */
   return num;
}

/****
 *
 * LeString(): L um string via teclado deixando o buffer
 *             zerado
 *
 * Parmetros:
 *      ar[] (sada): array que armazenar o string lido
 *      nElementos (entrada): nmero mximo de elementos que
 *                            sero armazenados no array,
 *                            incluindo '\0'
 *
 * Retorno: O nmero de caracteres que o usurio digitou a
 *          mais, sem incluir '\n'
 *
 ****/
int LeString(char *ar, int nElementos)
{
   char *p; /* Usado para remover '\n' do string */
   int   nCarDeixados = 0; /* Conta o nmero de     */
                           /* caracteres excedentes */
                           /* no buffer             */

      /* L no mximo nElementos - 1 caracteres ou at   */
      /* encontrar '\n'. Quando fgets() retorna NULL,    */
      /* ocorreu erro ou tentativa de leitura alm do    */
      /* final do arquivo. Nesses casos, o lao continua */
      /* efetuando novas tentativas de leitura.          */
   while ( fgets(ar, nElementos, stdin) == NULL )  {
      printf( "\a\n\t>>> %s", "Erro de leitura. "
              "Tente novamente\n\t> " );
   }

      /* Faz p apontar para o caractere que      */
      /* antecede o caractere terminal do string */
   p = strchr(ar, '\0') - 1;

      /* Se o caractere '\n' foi lido, remove-o do   */
      /* string. Caso contrrio, remove-o do buffer. */
   if (*p == '\n') {
         /* Caractere '\n' faz parte do string */
      *p = '\0'; /* Sobrescreve '\n' com '\0' */
   } else { /* Usurio digitou caracteres demais */
      nCarDeixados = LimpaBuffer();
   }

      /* Retorna o nmero de caracteres que o usurio */
      /* digitou alm do esperado, sem incluir '\n'   */
   return nCarDeixados;
}

void ExibeRegistro(const tRegistro *ptrRegistro)
{
   printf("Nome: %s\n", ptrRegistro->nome);
   printf( "Nascimento: %d/%d/%d\n",
           ptrRegistro->dia, ptrRegistro->mes, ptrRegistro->ano );
}

tRegistro *LeRegistro4(void)
{
   static tRegistro registro;

   printf("\nNome (max = %d letras): ", MAX_NOME - 1);
   LeString(registro.nome, MAX_NOME);

   printf("\nDia de nascimento: ");
   registro.dia = LeInteiro();

   printf("\nMes de nascimento: ");
   registro.mes = LeInteiro();

   printf("\nAno de nascimento: ");
   registro.ano = LeInteiro();

   return &registro;
}

int main(void)
{
   tRegistro *ptrPessoa1, *ptrPessoa2;

   ptrPessoa1 = LeRegistro4();
   ptrPessoa2 = LeRegistro4();

   printf("\n\n***** Dados da primeira pessoa *****\n\n");
   ExibeRegistro(ptrPessoa1);

   printf("\n\n***** Dados da segunda pessoa *****\n\n");
   ExibeRegistro(ptrPessoa2);

   return 0;
}

/******************* Exemplo de Execuo *******************

Nome (max = 29 letras): Jose da Silva

Dia de nascimento: 25

Mes de nascimento: 12

Ano de nascimento: 1971

Nome (max = 29 letras): Manoel da Silva

Dia de nascimento: 22

Mes de nascimento: 2

Ano de nascimento: 1980


***** Dados da primeira pessoa *****

Nome: Manoel da Silva
Nascimento: 22/2/1980


***** Dados da segunda pessoa *****

Nome: Manoel da Silva
Nascimento: 22/2/1980

***********************************************************/
