/****
 *
 * Ttulo: Ordenao de Listas Simplesmente Encadeadas
 *
 * Autor: Ulysses de Oliveira
 *
 * Data de Criao: 16/04/2017
 * ltima modificao: 16/04/2017
 *
 * Descrio: Exemplo de ordenao de listas encadeadas
 *
 * Entrada: Nada
 *
 * Sada: V. Exemplo de Execuo
 *
 ****/

/*********************** Includes *************************/

#include <stdio.h>
#include <stdlib.h>
#include <time.h>

/****************** Definies de Tipos *******************/

typedef struct no {
           int        dado;
           struct no *proximo;
        } tNo, *tLista;

/************************* Aluses ************************/

extern void OrdenaLista(tNo *lista);
extern tLista CriaLista(int n, int m);
extern void ExibeLista(tLista lista);

/***************** Definies de Funes ******************/

/****
 *
 * OrdenaLista(): Ordena uma lista encadeada
 *
 * Parmetros:
 *    lista (entrada/sada) - a lista encadeada que ser ordenada
 *
 * Retorno: Nada
 *
 ****/
void OrdenaLista(tNo *lista)
{
   tNo *p, *q;
   int  aux, ordenada = 0;

      /* Se a lista estiver vazia ou tiver apenas */
      /* um elemento, ela est ordenada           */
   if (!lista || !lista->proximo)
      return;

   while (!ordenada) {
      ordenada = 1;

      p = lista;
      q = lista->proximo;

      while (q) {
            /* Compara contedos dos ns apontados por p e q */
         if (p->dado > q->dado) {
               /* Ns esto fora de ordem*/
            ordenada = 0;

               /* Troca os contedos dos ns */
            aux = p->dado;
            p->dado = q->dado;
            q->dado = aux;
         }

            /* Passa para os ns seguintes  */
         p = q;
         q = q->proximo;
      }
   }
}

/****
 *
 * CriaLista(): Cria uma lista encadeada com contedos aleatrios
 *
 * Parmetros:
 *      n (entrada) - nmero de ns da lista
 *      m (entrada) - maior valor de contedo
 *
 * Retorno: Endereo da lista criada
 *
 ****/
tLista CriaLista(int n, int m)
{
   tLista lista = NULL, p;
   int    i;

   srand(time(NULL));

   for (i = 0; i < n; ++i) {
      p = malloc(sizeof(int));

      p->dado = rand()%m;
      p->proximo = lista;
      lista = p;
   }

   return lista;
}

/****
 *
 * ExibeLista(): Exibe na tela o contedo do tipo int de cada n
 *               de uma lista encadeada
 *
 * Parmetros:
 *      lista (entrada) - a lista a ser exibida
 *
 * Retorno: Nada
 *
 ****/
void ExibeLista(tLista lista)
{
   while (lista) {
       printf("%d\t", lista->dado);
       lista = lista->proximo;
   }

   putchar('\n');
}

/****
 *
 * main(): Testa a funo OrdenaLista()
 *
 * Parmetros: Nenhum
 *
 * Retorno: Zero
 *
 ****/
int main(void)
{
   tLista lista;

   lista = CriaLista(10, 20);

   printf("\n\t\t\t>>> Antes da ordenacao <<<\n\n");

   ExibeLista(lista);

   OrdenaLista(lista);

   printf("\n\t\t\t>>> Depois da ordenacao <<<\n\n");

   ExibeLista(lista);

   return 0;
}

/******************* Exemplo de Execuo ******************

                        >>> Antes da ordenacao <<<

4       17      15      19      14      17      6       17      2       8

                        >>> Depois da ordenacao <<<

2       4       6       8       14      15      17      17      17      19

**********************************************************/

