/****
 *
 * Exemplo: Listas Generalizadas
 *
 * Sees: 7.2
 *
 * Descrio: Demonstra o uso de listas generalizadas
 *
 ****/

/*********************** Includes *************************/

#include <stdio.h>   /* Funes de Entrada/Sada     */
#include <stdlib.h>  /* Funes de alocao dinmica */

/****************** Definies de Tipos *******************/

typedef enum {ATOMO, LISTA} tIndicador;

typedef union {
           char         atomo;
           struct  no  *lista;
        } tConteudo;

typedef struct no {
           tIndicador  indicador;
           tConteudo   conteudo;
           struct  no *proximo;
        } tNoG, *tListaG;

/************************* Aluses ************************/

extern tListaG CopiaLista(tListaG);
extern int SaoIguais(tListaG, tListaG);
extern int Prof(tListaG);
extern void  ExibeListaG(tListaG);

/***************** Definies de Funes ******************/

/****
 *
 * CopiaLista(): Retorna um ponteiro para uma lista
 *               que  uma duplicata de L
 *
 * Parmetros:
 *      L (entrada) - aponta para uma lista generalizada
 *                    no recursiva
 *
 * Retorno: A lista resultante da cpia
 *
 ****/
tListaG CopiaLista(tListaG  L)
{
   tListaG ptr = NULL;

   if (L) {
      ptr = malloc(sizeof(tNoG));
      if (L->indicador == ATOMO)
         ptr->conteudo.atomo = L->conteudo.atomo;
      else
         ptr->conteudo.lista = CopiaLista(L->conteudo.lista);
      ptr->proximo = CopiaLista(L->proximo);
      ptr->indicador = L->indicador;
   }

   return ptr;
}

/****
 *
 * SaoIguais(): Verifica se duas listas generalizadas
 *              so iguais
 *
 * Parmetros:
 *      L1, L2 (entrada) - listas generalizadas no-recursivas
 *
 * Retorno: 1 se L1 e L2 so iguais ou 0 caso contrrio
 *
 ****/
int SaoIguais(tListaG  L1, tListaG  L2)
{
   int resultado = 0;

   if (!L1 && !L2)
      resultado = 1;
   else if (L1 && L2)
      if (L1->indicador == L2->indicador) {
         if (L1->indicador == ATOMO)
             resultado = (L1->conteudo.atomo ==
                          L2->conteudo.atomo);
         else
             resultado = SaoIguais(L1->conteudo.lista,
                                   L2->conteudo.lista);

         if (resultado)
             resultado = SaoIguais(L1->proximo, L2->proximo);
      }

   return resultado;
}

/****
 *
 * Prof(): Calcula a profundidade de uma lista generalizada
 *
 * Parmetros:
 *      L (entrada) - a lista cuja profundidade ser calculada
 *
 * Retorno: A profundidade da lista recebida como parmetro
 *
 ****/
int Prof(tListaG  L)
{
   int     max, resultado;
   tListaG ptr;

   if (!L)
         /* A lista vazia tem profundidade igual a zero */
      return 0;

   max = 0;

   ptr = L;

      /* Calcula a profundidade de cada n */
   while (ptr) {
         /* Profundidade do n corrente */
      if (ptr->indicador == ATOMO)
         resultado = 0;
      else                     /* Idem */
         resultado = Prof(ptr->conteudo.lista);

      if (resultado > max)
            /* Atualiza o maior valor encontrado at aqui */
         max = resultado;

      ptr = ptr->proximo;
   }

   return max + 1;
}

/****
 *
 * ExibeListaG(): Exibe na tela os ns de uma lista
 *                generalizada no recursiva
 *
 * Parmetros:
 *      p (entrada) - a lista que ser exibida
 *
 * Retorno: Nada
 *
 ****/
void  ExibeListaG(tListaG  p)
{
   if (!p) {
      printf("()");
      return;
   }

   putchar('(');

   while (p)  {
      if (p->indicador == ATOMO)
         putchar(p->conteudo.atomo);
      else
         ExibeListaG(p->conteudo.lista);

      p = p->proximo;
   }

   putchar(')');
}

/****
 *
 * main(): Testa operaes sobre listas generalizAdas
 *
 * Parmetros: Nenhum
 *
 * Retorno: Zero
 *
 ****/
int main (void)
{
   tListaG lista, ptr1, ptr2, lista2;

   /*                                    */
   /* Construo da lista: A = (a,(b,c)) */
   /*                                    */

   lista = malloc(sizeof(tNoG));   /* Cria o primeiro n */
   lista->indicador = ATOMO;   /* Este n contm um tomo */
   lista->conteudo.atomo = 'a';

      /* Cria o segundo n */
   ptr1 = malloc(sizeof(tNoG));
      /* Faz primeiro n apontar para o segundo */
   lista->proximo = ptr1;
      /* Este n aponta para uma sublista */
   ptr1->indicador = LISTA;
      /* Este n  o ltimo da lista */
   ptr1->proximo = NULL;

   /*                                    */
   /* Criao da sublista do segundo n */
   /*                                    */

      /* Cria o primeiro n da sublista */
   ptr2 = malloc(sizeof(tNoG));
      /* Este n contm um tomo */
   ptr2->indicador = ATOMO;
   ptr2->conteudo.atomo = 'b';

      /* Faz segundo n da lista apontar */
      /* para o primeiro n da sublista  */
   ptr1->conteudo.lista = ptr2;

      /* Cria o segundo n da sublista */
   ptr1 = malloc(sizeof(tNoG));
      /* Faz o primeiro n da sublista */
      /* apontar para o segundo        */
   ptr2->proximo = ptr1;

      /* Este n contm um tomo */
   ptr1->indicador = ATOMO;
   ptr1->conteudo.atomo = 'c';
      /* Este n  o ltimo da sublista */
   ptr1->proximo = NULL;

   printf("\nPrimeira lista:\n");
   ExibeListaG(lista);

   lista2 = CopiaLista(lista);

   printf("\n\nSegunda lista:\n");
   ExibeListaG(lista2);

   printf( "\n\nProfundidade da primeira lista: %d",
           Prof(lista) );
   printf( "\n\nProfundidade da segunda lista: %d\n",
           Prof(lista2) );

   if (SaoIguais(lista, lista2))
      printf("\nAs listas sao iguais\n");
   else
      printf("\nAs listas sao diferentes\n");

   return 0;
}

/****************** Resultado do Programa *****************

Primeira lista:
(a(bc))

Segunda lista:
(a(bc))

A profundidade da primeira lista e': 2

A profundidade da segunda lista e': 2

As listas sao iguais

**********************************************************/


