/****
 *
 * Ttulo: O Problema da Bandeira Holandesa
 *
 * Autor: Ulysses de Oliveira
 *
 * Data de Criao: 16/05/2017
 * ltima modificao: 16/05/2017
 *
 * Descrio: Divide um array contendo apenas elementos
 *            com valores 0, 1 e 2 em  trs sees: a primeira
 *            contm elementos iguais a 0, a segunda contm iguais
 *            a 1 e a terceira contm elementos iguais a 2
 *
 * Entrada: Nada
 *
 * Sada: V. Exemplo de Execuo
 *
 ****/

/*********************** Includes *************************/

#include <stdio.h>
#include <stdlib.h>

/************************* Aluses ************************/

extern void Separa(int lista[], int n);
extern void Separa(int lista[], int n);
extern void ExibeLista(int lista[], int n);

/***************** Definies de Funes ******************/

/****
 *
 * Troca(): Troca os valores de duas variveis do tipo int
 *
 * Parmetros:
 *      x, y (entrada/sada) - endereos das variveis que
 *                             tero seus valores trocados
 *
 * Retorno: Nada
 *
 ****/
void Troca(int *a, int *b)
{
   int aux = *a;

   *a = *b;
   *b = aux;
}

/****
 *
 * Separa(): Divide um array em trs parties: uma contendo
 *           valores iguais a 0, outra com valores iguais a
 *           1 e a ltima com valores iguais a 2
 *
 * Parmetros:
 *      lista (entrada) - a lista
 *      n (entrada) - nmero de elementos da lista
 *
 * Retorno: Nada
 *
 * Observao: Os nicos valores permitidos no array so 0, 1 e 2
 *
 ****/
void Separa(int lista[], int n)
{
   int inf = 0,
       sup = n - 1,
       meio = 0;

      /* Coloca cada elemento em sua devida partio */
   while (meio <= sup) {
      switch (lista[meio]) {
         case 0:
            Troca(&lista[inf], &lista[meio]);
            ++inf;
            ++meio;
            break;
         case 1:
            meio++;
            break;
         case 2:
            Troca(&lista[meio], &lista[sup]);
            --sup;
            break;
         default:
            printf("\nValor de array diferente de 0, 1 ou 2\n");
            exit(1);
      }
   }
}

/****
 *
 * ExibeLista(): Exibe na tela o contedo do tipo int de cada n
 *               de uma lista encadeada
 *
 * Parmetros:
 *      lista (entrada) - a lista v
 *
 * Retorno: Nada
 *
 ****/
void ExibeLista(int lista[], int n)
{
   int i;

   printf("\n{ ");

   for (i = 0; i < n - 1; ++i)
      printf("%d, ", lista[i]);

   printf("%d }\n", lista[i]);
}

/****
 *
 * main(): Testa a funo OrdenaLista()
 *
 * Parmetros: Nenhum
 *
 * Retorno: Zero
 *
 ****/
int main(void)
{
   int ar[] = {0, 1, 1, 0, 1, 2, 1, 2, 0, 0, 0, 1},
       tam = sizeof(ar)/sizeof(ar[0]);

   printf("\n>>> Lista original <<<\n");

   ExibeLista(ar, tam);

   printf("\n>>> Lista separada <<<\n");

   Separa(ar, tam);

   ExibeLista(ar, tam);

   return 0;
}

/******************* Exemplo de Execuo ******************

>>> Lista original <<<

{ 0, 1, 1, 0, 1, 2, 1, 2, 0, 0, 0, 1 }

>>> Lista separada <<<

{ 0, 0, 0, 0, 0, 1, 1, 1, 1, 1, 2, 2 }

**********************************************************/

