/****
 *
 * Ttulo: rvores Binrias 2
 *
 * Autor: Ulysses de Oliveira
 *
 * Data de Criao: 16/06/2012
 * ltima modificao: 16/06/2012
 *
 * Descrio: Testa operaes sobre rvores
 *
 * Entrada: No h
 *
 * Sada: v. exemplo de execuo
 *
 ****/

/*********************** Includes *************************/

#include <stdio.h>   /* Funes de entrada e sada  */
#include "Arvore.h"  /* Declaraes do tipo rvore  */

/***************** Definies de Funes ******************/

/****
 *
 * ExibeDado(): Exibe na tela o contedo do campo dado
 *                de um n de uma rvore binria
 *
 * Parmetros:
 *      item (entrada) - item de dado que ser impresso
 *
 * Retorno: Nenhum
 *
 * Observao: Assume-se que o tipo do contedo a ser
 *             impresso  char
 *
 ****/
static void ExibeDado(tConteudo item)
{
   putchar((int) item);
}

/****
 *
 * ExibeDadoFib(): Exibe na tela o contedo do campo dado
 *                 de um n de uma rvore binria de Fibonacci
 *
 * Parmetros:
 *      dado (entrada) - item de dado que ser impresso
 *
 * Retorno: Nenhum
 *
 * Observao: Assume-se que o tipo do contedo a ser
 *             impresso  int
 *
 ****/
static void ExibeDadoFib(tConteudo item)
{
   printf("%d", (int) item);
}

/****
 *
 * main(): Testa as operaes sobre rvores
 *
 * Parmetros: Nenhum
 *
 * Retorno: Zero
 *
 ****/
int main (void)
{
   tArvore arvore, copia, pNo, aFibonacci;

/********
*
* rvore a ser construda como exemplo:
*
*               A
*              / \
*              /   \
*             /     \
*            B       C
*           /       / \
*          /       /   \
*         D       E     F
*          \     / \
*           \   /   \
*            G  H    I
*
*********/

   /***********************************/
   /*                                 */
   /* Primeiro nvel da rvore (raiz) */
   /*                                 */
   /***********************************/

   arvore = ConstroiNo('A');

   /***************************/
   /*                         */
   /* Segundo nvel da rvore */
   /*                         */
   /***************************/

      /* Constri filho da esquerda */
   FilhoEsquerda(arvore, 'B');
      /* Constri filho da direita */
   FilhoDireita(arvore, 'C');

   /****************************/
   /*                          */
   /* Terceiro nvel da rvore */
   /*                          */
   /****************************/

   FilhoEsquerda(arvore->esquerda, 'D');
   FilhoEsquerda(arvore->direita, 'E');
   FilhoDireita(arvore->direita, 'F');

      /* Guarda o endereo do n 'F' para */
      /* testar funo NivelDoNo()        */
   pNo = arvore->direita->direita;

   /**************************/
   /*                        */
   /* Quarto nvel da rvore */
   /*                        */
   /**************************/

   FilhoDireita(arvore->esquerda->esquerda, 'G');
   FilhoEsquerda(arvore->direita->esquerda, 'H');
   FilhoDireita(arvore->direita->esquerda, 'I');

      /* Exibe a rvore */
   printf("\n\t*** Arvore ***\n\n");
   Exibe(arvore, ExibeDado);

      /* Calcula a profundidade da rvore */
   printf( "\n>>> Profundidade da arvore: %d\n",
           Profundidade(arvore) );

      /* Calcula  nmero de ns da rvore */
   printf( "\n>>> Numero de nos da arvore: %d\n",
           NumeroDeNos(arvore) );

      /* Caminha em pr-ordem na rvore    */
      /* imprimindo o contedo de cada n. */
   printf("\n>>> Caminhamento em ordem prefixa: ");
   Prefixo(arvore, ExibeDado);

       /* Faz uma cpia da rvore original */
   copia = CopiaArvore(arvore);

      /* Exibe a cpia da rvore */
   printf("\n\n\t*** Copia da arvore ***\n\n");
   Exibe(copia, ExibeDado);

      /* Encontra o nvel do n 'F' na rvore */
   printf( "\n>>> Nivel do no %c na arvore: %d", pNo->conteudo,
           NivelDoNo(arvore, pNo) );

      /* Tenta encontrar o nvel do n 'F' na cpia */
   printf( "\n>>> Nivel do no %c na copia da arvore: %d",
           pNo->conteudo, NivelDoNo(copia, pNo) );

   for (int i = 0; i <= 4; ++i) {
         /* Cria uma rvore de Fibonacci */
      aFibonacci = ArvoreFib(i);

         /* Exibe a rvore de Fibonacci */
      printf( "\n\n   *** Arvore de Fibonacci de grau "
              "%d ***\n\n", i );
      Exibe(aFibonacci, ExibeDadoFib);

         /* Profundidade da rvore de Fibonacci */
      printf( "\n>>> Profundidade da arvore de Fibonacci:"
              " %d\n", Profundidade(aFibonacci) );

         /* Nmero de ns da rvore de Fibonacci */
      printf( ">>> Numero de nos da arvore de Fibonacci: "
              "%d\n", NumeroDeNos(aFibonacci) );
   }

   return 0;
}

/****************** Resultado do Programa ******************

        *** Arvore ***

A
        C
                F
                E
                        I
                        H
        B
                D
                        G

>>> Profundidade da arvore: 4

>>> Numero de nos da arvore: 9

>>> Caminhamento em ordem prefixa: ABDGCEHIF

        *** Copia da arvore ***

A
        C
                F
                E
                        I
                        H
        B
                D
                        G

>>> Nivel do no F na arvore: 3
>>> Nivel do no F na copia da arvore: 0

   *** Arvore de Fibonacci de grau 0 ***


>>> Profundidade da arvore de Fibonacci: 0
>>> Numero de nos da arvore de Fibonacci: 0


   *** Arvore de Fibonacci de grau 1 ***

1

>>> Profundidade da arvore de Fibonacci: 1
>>> Numero de nos da arvore de Fibonacci: 1


   *** Arvore de Fibonacci de grau 2 ***

2
        1

>>> Profundidade da arvore de Fibonacci: 2
>>> Numero de nos da arvore de Fibonacci: 2


   *** Arvore de Fibonacci de grau 3 ***

3
        1
        2
                1

>>> Profundidade da arvore de Fibonacci: 3
>>> Numero de nos da arvore de Fibonacci: 4


   *** Arvore de Fibonacci de grau 4 ***

4
        2
                1
        3
                1
                2
                        1

>>> Profundidade da arvore de Fibonacci: 4
>>> Numero de nos da arvore de Fibonacci: 7

***********************************************************/








