/****
 *
 * Ttulo: rvores Binrias
 *
 * Autor: Ulysses de Oliveira
 *
 * Data de Criao: 16/06/2012
 * ltima modificao: 16/06/2012
 *
 * Descrio: Testa operaes sobre rvores
 *
 * Entrada: No h
 *
 * Sada: v. exemplo de execuo
 *
 ****/

/*********************** Includes *************************/

#include <stdio.h>   /* Funes de Entrada/Sada       */
#include "Arvore.h"  /* Operaes com rvores binrias */

/***************** Definies de Funes ******************/

/****
 *
 * ExibeDado(): Exibe na tela o contedo do campo dado
 *                de um n de uma rvore binria
 *
 * Parmetros:
 *      item (entrada) - contedo que ser impresso
 *
 * Retorno: Nenhum
 *
 * Observao: Assume-se que o tipo do contedo a ser
 *             impresso  char.
 *
 ****/
static void ExibeDado(tConteudo item)
{
   putchar((int) item);
}

/****
 *
 * main(): Testa as operaes sobre rvores
 *
 * Parmetros: Nenhum
 *
 * Retorno: Zero.
 *
 ****/
int main (void)
{
   tArvore arvore;

/********
 *
 * rvore a ser construda como exemplo:
 *
 *                 A
 *                / \
 *              /    \
 *             /      \
 *            B        C
 *           /        / \
 *          /        /   \
 *         D        E     F
 *          \      / \
 *           \    /   \
 *            G  H     I
 *
 *********/

   arvore = ConstroiNo('A'); /* Primeiro nvel (raiz) */

      /* Construo do segundo nvel da rvore */
   FilhoEsquerda(arvore, 'B'); /* Constri filho esquerdo */
   FilhoDireita(arvore, 'C');  /* Constri filho direito  */

      /* Construo do terceiro nvel da rvore */
   FilhoEsquerda(arvore->esquerda, 'D');
   FilhoEsquerda(arvore->direita, 'E');
   FilhoDireita(arvore->direita, 'F');

      /* Construo do quarto nvel da rvore */
   FilhoDireita(arvore->esquerda->esquerda, 'G');
   FilhoEsquerda(arvore->direita->esquerda, 'H');
   FilhoDireita(arvore->direita->esquerda, 'I');

      /* Exibe a rvore */
   printf("\n\t*** Arvore ***\n\n");
   Exibe(arvore, ExibeDado);

      /* Caminha em ordem infixa na rvore */
      /* imprimindo o contedo de cada n  */
   printf("\n\n>>> Caminhamento em ordem infixa: ");
   Inordem(arvore, ExibeDado);

      /* Caminha em ordem infixa na rvore */
      /* imprimindo o contedo de cada n  */
      /* usando a funo Inordem2.         */
   printf( "\n\n>>> Caminhamento em ordem infixa usando "
            "Inordem2(): " );
   Inordem2(arvore, ExibeDado);

   return 0;
}

/****************** Resultado do Programa ******************

        *** Arvore ***

A
        C
                F
                E
                        I
                        H
        B
                D
                        G


>>> Caminhamento em ordem infixa: DGBAHEICF

>>> Caminhamento em ordem infixa usando Inordem2(): DGBAHEICF

***********************************************************/
