/****
 *
 * Ttulo: rvores Binrias Costuradas
 *
 * Autor: Ulysses de Oliveira
 *
 * Data de Criao: 16/06/2012
 * ltima modificao: 16/06/2012
 *
 * Descrio: Testa operaes sobre rvores
 *
 * Entrada: No h
 *
 * Sada: v. exemplo de execuo
 *
 ****/

/*********************** Includes *************************/

#include <stdio.h>   /* Funes de Entrada e Sada  */
#include "ArvoreC.h" /* rvores binrias costuradas */

/***************** Definies de Funes ******************/

/****
 *
 * ExibeDado(): Exibe na tela o contedo do campo dado
 *                de um n de uma rvore binria
 *
 * Parmetros:
 *      item (entrada) - item de dado que ser impresso
 *
 * Retorno: Nenhum
 *
 * Observao: Assume-se que o tipo do contedo a ser
 *             impresso  char.
 *
 ****/
static void ExibeDado(tConteudo item)
{
   putchar((int) item);
}

/****
 *
 * main(): Testa as operaes sobre rvores.
 *
 * Parmetros: Nenhum
 *
 * Retorno: Zero, se no ocorrer nenhum erro.
 *          Um valor diferente de zero em caso contrrio.
 *
 ****/
int main (void)
{
   tArvoreC arvore;

/********
 *
 * rvore a ser construda como exemplo:
 *
 *                 A
 *                / \
 *              /   \
 *             /     \
 *            B       C
 *           /       / \
 *          /       /   \
 *         D       E     F
 *          \     / \
 *           \   /   \
 *            G  H    I
 *
 *********/

   /***********************************/
   /*                                 */
   /* Primeiro nvel da rvore (raiz) */
   /*                                 */
   /***********************************/

   arvore = ConstroiNoC('A');

   /***************************/
   /*                         */
   /* Segundo nvel da rvore */
   /*                         */
   /***************************/

      /* Constri filho da esquerda */
   FilhoEsquerdaC(arvore, 'B');
      /* Constri filho da direita */
   FilhoDireitaC(arvore, 'C');

   /****************************/
   /*                          */
   /* Terceiro nvel da rvore */
   /*                          */
   /****************************/

   FilhoEsquerdaC(arvore->esquerda, 'D');
   FilhoEsquerdaC(arvore->direita, 'E');
   FilhoDireitaC(arvore->direita, 'F');

   /**************************/
   /*                        */
   /* Quarto nvel da rvore */
   /*                        */
   /**************************/

   FilhoDireitaC(arvore->esquerda->esquerda, 'G');
   FilhoEsquerdaC(arvore->direita->esquerda, 'H');
   FilhoDireitaC(arvore->direita->esquerda, 'I');

      /* Imprime a rvore */
   printf("\n\t*** Arvore ***\n\n");
   ExibeC(arvore, ExibeDado);

      /* Caminha em ordem prefixa na rvore */
      /* imprimindo o contedo de cada n   */
   printf("\n\n>>> Caminhamento em ordem prefixa: ");
   PreordemC(arvore, ExibeDado);

      /* Caminha em ordem infixa na rvore */
      /* imprimindo o contedo de cada n  */
   printf("\n\n>>> Caminhamento em ordem infixa: ");
   InordemC(arvore, ExibeDado);

      /* Caminha em ordem sufixa na rvore */
      /* imprimindo o contedo de cada n  */
   printf("\n\n>>> Caminhamento em ordem sufixa: ");
   PosordemC(arvore, ExibeDado);

   return 0;
}

/****************** Resultado do Programa ******************

        *** Arvore ***

A
        C
                F
                E
                        I
                        H
        B
                D
                        G


>>> Caminhamento em ordem prefixa: ABDGCEHIF

>>> Caminhamento em ordem infixa: DGBAHEICF

>>> Caminhamento em ordem sufixa: GDBHIEFCA

***********************************************************/
